@@ -1280,7 +1280,7 @@ class SplitDataset():
     # dist : int, number of data to be leftover between two adjacent sec, default is 0
     # returns data, label and indices for train, validation and test data sets as pydict
     def sequence_split(self, seq_len, val_ratio = .15, data_stride = 1, label_shift = 1,
-                       split_method = 'multiple_train_val', sec = 1, dist = 0):
+                       split_method = 'multiple_train_val', sec = 1, dist = 0, inference = False):
         # split_method
         if split_method == 'train_val':
             if sec == 1: sec = 1
@@ -1294,6 +1294,14 @@ class SplitDataset():
         val_len_ps = int(val_ratio / (1 - self.test_ratio) * data_per_sec)      # number of validation indices per section
         tr_len_ps = int(data_per_sec - val_len_ps)       # number of train indices per section
         outdata = {x : {y : {'section_%d'%(i + 1) : [] for i in range(sec)} for y in self.dataset} for x in ['train', 'validation', 'test']}
+        if inference:
+            outdata = {'inference' : {m : [] for m in ['data', 'label', 'index']}}
+            for st in range(self.data_shape[0] - seq_len + 1):
+                outdata['inference']['data'].append(self.dataset['data'][st : st + seq_len])
+                outdata['inference']['label'].append(self.dataset['label'][st + seq_len - 1 + label_shift])
+                outdata['inference']['index'].append(self.dataset['index'][st + seq_len - 1])
+            for k in outdata['inference']: outdata['inference'][k] = np.array(outdata['inference'][k])
+            return outdata
         if test_fl:
             # test data starting indices
             sts = np.array([self.data_shape[0] - seq_len - label_shift - i * data_stride for i in range(test_len)])
@@ -1325,3 +1333,76 @@ class SplitDataset():
                 if sec == 1 or nm == 'test':
                     outdata[nm][k] = outdata[nm][k]['section_1']
         return outdata
+
+   
+
+# params : dict of lists containing choices for each of the param keys
+# mode : str, search mode; avaialble modes {'random', 'grid'}, default is 'random'
+# find : str, minimizing or maximizing the score ? ; available {'min', 'max'}, default 'min'
+# iteration : int, number of iterations to be done, default is None
+# top : int, number of top scores to be stored; default is 3
+# rand_it_perc : float (0 ~ 1), indirectly sets the total number of iteration in 'random' model if iteration is not given, default is .5
+# shuffle_queue : bool, whether to shuffle the parameter queue or not, default is True
+
+class paramOptimizer():
+    
+    def __init__(self, params, mode = 'random', find = 'min', iteration = None, top = 3, rand_it_perc = .5, shuffle_queue = True):
+        self.params = params
+        self.paramlen = {p : len(params[p]) for p in params}
+        self.queue = self.get_queue()
+        self.total_it = len(self.queue)
+        self.mode = mode
+        if self.mode == 'random':
+            if iteration is None: self.iteration = int(self.total_it * rand_it_perc)
+            else: self.iteration = iteration
+            shuffle_queue = True
+        elif self.mode == 'grid': self.iteration = self.total_it
+        if shuffle_queue: random.shuffle(self.queue)
+        self.queue = pd.DataFrame(self.queue)
+        self.queue['score'] = np.nan
+        self.queue_cnt = 0
+        self.top = top
+        self.ascending = True if find == 'min' else False
+        self.tops = pd.DataFrame(index = [i for i in range(self.top + 1)], columns = ['queue_idx', 'score'])
+        self.storage = []
+    
+    def get_queue(self,):
+        counts = [[p for p in self.params], [0 for _ in self.params], [len(self.params[p]) for p in self.params]]
+        total_param = len(self.params)
+        records = []
+        while True:
+            record = {p : self.params[p][i] for p, i in zip(counts[0], counts[1])}
+            records.append(record)
+            counts[1][0] += 1
+            for pnum in range(1, total_param):
+                if counts[1][pnum - 1] == counts[2][pnum - 1]: 
+                    counts[1][pnum] += 1
+                    counts[1][pnum - 1] = 0
+            if counts[1][-1] == counts[2][-1] : break
+        return records
+    
+    # gives the selected parameter
+    def get_param(self,):
+        return dict(self.queue.drop('score', axis = 1).iloc[self.queue_cnt])
+    
+    # this function takes in score value on what optimization is done
+    # score : the score value
+    # element : any additional storage element like model or any other variable etc.
+    def set_score(self, score, element = ''):
+        self.queue['score'].iloc[self.queue_cnt] = score
+        if self.queue_cnt < self.top + 1:
+            self.tops.iloc[self.queue_cnt] = [self.queue_cnt, score]
+            self.storage.append(element)
+        else:
+            self.tops.iloc[self.top] = [self.queue_cnt, score]
+            self.storage[-1] = element
+            self.tops.sort_values('score', ascending = self.ascending, inplace = True)
+            self.storage = [self.storage[i] for i in self.tops.index]
+            self.tops.reset_index(drop = True, inplace = True)
+        self.queue_cnt += 1
+        if self.queue_cnt == self.iteration: return True
+        else: return False
+    
+    # this function provides the best parameter set in the end (can also be used at any time instance in stead of end)
+    def best(self,):
+        return self.queue.iloc[self.tops['queue_idx'].values[:-1]].reset_index(drop = True), self.storage[:-1]
